from typing import Final

SQL_COUNT_TILES: Final[str] = 'select count(*) as count from tiles;'
SQL_COUNT_TILES_BY_Z: Final[str] = 'select zoom_level as zoom_level, count(*) as count from tiles group by zoom_level order by zoom_level ASC;'



from collections import namedtuple
from typing import List
import json
from tileset_analyzer.entities.level_count import LevelCount

#LevelCount = namedtuple('LevelCount', ['z', 'count'])

class CustomEncoder(json.JSONEncoder):
    def default(self, o):
            return o.__dict__

class TilesetAnalysisResult:
    def __init__(self):
        pass
    
    def set_count_tiles_total(self, num: int):
        self.count_tiles_total = num

    def set_count_tiles_by_z(self, level_counts: List[LevelCount]):
        self.count_tiles_by_z = level_counts

    
    def get_json(self):
        return json.dumps(self.__dict__, indent=4, cls=CustomEncoder, sort_keys=True,
                      separators=(',', ': '), ensure_ascii=False)


import abc
from collections import namedtuple
from sqlite3 import Connection
from typing import List
from tileset_analyzer.data_source.mbtiles.sqllite_utils import create_connection
from tileset_analyzer.data_source.tile_source import TileSource
from tileset_analyzer.entities.tileset_analysis_result import LevelCount, TilesetAnalysisResult
from tileset_analyzer.data_source.mbtiles.sql_queries import SQL_COUNT_TILES, SQL_COUNT_TILES_BY_Z


class MBTileSource(TileSource):
    def __init__(self, src_path: str):
        self.conn = create_connection(src_path)

    def count_tiles(self) -> int:
        cur = self.conn.cursor()
        cur.execute(SQL_COUNT_TILES)
        count = cur.fetchone()[0]
        return count

    def count_tiles_by_z(self) -> List[LevelCount]:
        cur = self.conn.cursor()
        cur.execute(SQL_COUNT_TILES_BY_Z)
        rows = cur.fetchall()
        result = []
        for row in rows:
            result.append(LevelCount(row[0], row[1]))
        return result

    def analyze(self) -> TilesetAnalysisResult:
        result = TilesetAnalysisResult()
        result.set_count_tiles_total(self.count_tiles())
        result.set_count_tiles_by_z(self.count_tiles_by_z())
        return result



from fastapi import FastAPI
from fastapi.staticfiles import StaticFiles
import uvicorn


def start_api():
    app = FastAPI()
    app.mount("/api", StaticFiles(directory="tileset_analyzer/static/data"), name="api")
    app.mount("/", StaticFiles(directory="tileset_analyzer/static/ui", html=True), name="ui")
    
    uvicorn.run(app, host="0.0.0.0", port=8080)
    
